import 'package:puma_network/redux/api/apiAction.dart';
import 'package:puma_network/redux/api/apiErrorAction.dart';
import 'package:puma_network/redux/network/networkAction.dart';
import 'package:puma_network/redux/network/networkState.dart';
import 'package:puma_network/redux/response/responseAction.dart';
import 'package:redux/redux.dart';

final networkReducer = combineReducers<NetworkState>([
  TypedReducer<NetworkState, InitNetworkAction>(_initNetwork),
  TypedReducer<NetworkState, SetTokenNetworkAction>(_setToken),
  TypedReducer<NetworkState, SetBiometricTokenNetworkAction>(
      _setBiometricToken),
  TypedReducer<NetworkState, APIAction>(_addInNetworkCompletionStack),
  TypedReducer<NetworkState, ResponseAction>(_removeInNetworkCompletionStack),
  TypedReducer<NetworkState, ClearTokenNetworkAction>(_clearTokenNetworkAction),
  TypedReducer<NetworkState, ApiErrorAction>(_errorInNetworkCompletionStack),
  TypedReducer<NetworkState, ClearLastApiError>(_clearLastApiError),
  TypedReducer<NetworkState, SetImpersonateTokenNetworkAction>(
      _setImpersonateTokenNetwork),
]);

NetworkState _initNetwork(NetworkState state, InitNetworkAction action) {
  return state.rebuild((b) => b
    ..apiEndpoint = action.apiEndpoint
    ..https = action.https);
}

NetworkState _clearTokenNetworkAction(
    NetworkState state, ClearTokenNetworkAction action) {
  return state.rebuild((b) => b..token = "");
}

NetworkState _setToken(NetworkState state, SetTokenNetworkAction action) {
  return state.rebuild((b) => b..token = action.token);
}

NetworkState _setBiometricToken(
    NetworkState state, SetBiometricTokenNetworkAction action) {
  return state.rebuild((b) => b..token = action.token);
}

NetworkState _addInNetworkCompletionStack(
    NetworkState state, APIAction action) {
  if (action.networkCompletionKey == null) return state;
  return state.rebuild(
      (b) => b..networkCompletionStack[action.networkCompletionKey] = true);
}

NetworkState _removeInNetworkCompletionStack(
    NetworkState state, ResponseAction action) {
  return state.rebuild(
      (b) => b..networkCompletionStack.remove(action.networkCompletionKey));
}

NetworkState _errorInNetworkCompletionStack(
    NetworkState state, ApiErrorAction action) {
  return state.rebuild((b) => b
    ..networkCompletionStack.remove(action.networkCompletionKey)
    ..lastApiError.replace(action.error));
}

NetworkState _clearLastApiError(NetworkState state, ClearLastApiError action) {
  return state.rebuild((b) => b..lastApiError = null);
}

NetworkState _setImpersonateTokenNetwork(
    NetworkState state, SetImpersonateTokenNetworkAction action) {
  print("Hey!");
  return state
      .rebuild((b) => b..authImpersonateToken = action.authImpersonateToken);
}
