import 'dart:convert';

import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:puma_network/entities/apiError.dart';
import 'package:puma_network/serializers.dart';

part 'networkState.g.dart';

abstract class VirtualNetworkState {
  // ErrorState get errorState;
  NetworkState get networkState;
}

abstract class NetworkState
    implements Built<NetworkState, NetworkStateBuilder> {
  NetworkState._();
  factory NetworkState([updates(NetworkStateBuilder b)]) = _$NetworkState;

  static NetworkState initial() {
    return new NetworkState((b) => b
      ..apiEndpoint = "localhost:4200"
      ..https = true
      ..isLoading = false
      ..networkCompletionStack = MapBuilder<String, bool>()
      ..token = "");
  }

  String get apiEndpoint;
  bool get https;
  String get token;
  String? get authImpersonateToken;
  bool get isLoading;
  ApiError? get lastApiError;
  BuiltMap<String, bool> get networkCompletionStack;

  String toJson() {
    return json
        .encode(serializers.serializeWith(NetworkState.serializer, this));
  }

  static NetworkState? fromJson(jsonString) {
    return serializers.deserializeWith(
        NetworkState.serializer, json.decode(jsonString));
  }

  static Serializer<NetworkState> get serializer => _$networkStateSerializer;
}
